﻿#region License Information
/* HeuristicLab
 * Copyright (C) 2002-2019 Heuristic and Evolutionary Algorithms Laboratory (HEAL)
 *
 * This file is part of HeuristicLab.
 *
 * HeuristicLab is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * HeuristicLab is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HeuristicLab. If not, see <http://www.gnu.org/licenses/>.
 */
#endregion

using HEAL.Attic;
using HeuristicLab.Common;
using HeuristicLab.Core;
using HeuristicLab.Encodings.SymbolicExpressionTreeEncoding;
using System.Collections.Generic;

namespace HeuristicLab.Algorithms.EvolvmentModelsOfModels {
  [Item("IslandMap", "A map of models of models of models")]
  [StorableType("E4AB04B9-FD5D-47EE-949D-243660754F3A")]
  public class EMMIslandMap : EMMMapBase<ISymbolicExpressionTree> {
    #region conctructors
    [StorableConstructor]
    protected EMMIslandMap(StorableConstructorFlag _) : base(_) { }
    public EMMIslandMap() : this(1) { }
    public override IDeepCloneable Clone(Cloner cloner) {
      return new EMMIslandMap(this, cloner);
    }
    public EMMIslandMap(int k) : base(k) { ModelSet = new List<ISymbolicExpressionTree>(); }
    public EMMIslandMap(EMMIslandMap original, Cloner cloner) : base(original, cloner) { }
    // public EMMIslandMap(IRandom random, IEnumerable<ISymbolicExpressionTree> trees, int k, int neghboorNumber) : base(random, trees, k) { }
    //public EMMIslandMap(IRandom random, IEnumerable<ISymbolicExpressionTree> trees, string fileName = "Map.txt") : base(random, trees, fileName) { }
    #endregion
    #region MapApplayFunctions
    override public void CreateMap(IRandom random, int k) {
      K = EMModelsClusterizationAlgorithm.ApplyClusteringAlgorithm(random, Distances, ClusterNumber, k);
      MapSizeCheck();
      for (int i = 0; i < ModelSet.Count; i++) {
        Map[ClusterNumber[i]].Add(i);
      }
    }
    override public ISymbolicExpressionTree NewModelForInizializtionNotTree(IRandom random, out int cluster, out int treeNumber) {
      return NewModelForInizializtion(random, out cluster, out treeNumber);
    }
    #endregion

  }
}
